# Generated by Django 2.2.5 on 2019-09-20 13:45

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djeym', '0004_auto_20190330_1458'),
    ]

    operations = [
        migrations.AlterField(
            model_name='heatpoint',
            name='json_code',
            field=models.TextField(blank=True, default='{"type": "Feature", "id": 0, "geometry": {"type": "Point", "coordinates": []}, "properties": {"weight": 0}}', editable=False, verbose_name='JSON'),
        ),
        migrations.AlterField(
            model_name='placemark',
            name='json_code',
            field=models.TextField(blank=True, default='{"type": "Feature", "id": 0, "geometry": {"type": "Point", "coordinates": []}, "properties": {"balloonContentHeader": "", "balloonContentBody": "", "balloonContentFooter": "", "id": 0, "categoryID": 0, "subCategoryIDs": [], "iconName": ""}, "options": {"iconLayout": "default#image", "iconImageHref": "", "iconImageSize": [], "iconImageOffset": []}}', editable=False, verbose_name='JSON'),
        ),
        migrations.AlterField(
            model_name='polygon',
            name='json_code',
            field=models.TextField(blank=True, default='{"type": "Feature", "id": 0, "geometry": {"type": "Polygon", "coordinates": []}, "properties": {"balloonContentHeader": "", "balloonContentBody": "", "balloonContentFooter": "", "id": 0, "categoryID": 0}, "options": {"strokeWidth": 2, "strokeColor": "#1e98ff", "strokeOpacity": 0.9, "fillColor": "#1e98ff", "fillOpacity": 0.9}}', editable=False, verbose_name='JSON'),
        ),
        migrations.AlterField(
            model_name='polyline',
            name='json_code',
            field=models.TextField(blank=True, default='{"type": "Feature", "id": 0, "geometry": {"type": "LineString", "coordinates": []}, "properties": {"balloonContentHeader": "", "balloonContentBody": "", "balloonContentFooter": "", "id": 0, "categoryID": 0}, "options": {"strokeWidth": 5, "strokeColor": "#1e98ff", "strokeOpacity": 0.9}}', editable=False, verbose_name='JSON'),
        ),
    ]
