# Generated by Django 2.1.7 on 2019-03-21 09:53

from django.db import migrations, models
import django.db.models.deletion
import djeym.utils


class Migration(migrations.Migration):

    dependencies = [
        ('djeym', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='LoadIndicator',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('svg', models.FileField(help_text='https://github.com/genkosta/django-editor-ymaps/blob/master/Spinner.zip', null=True, upload_to=djeym.utils.make_upload_path, validators=[djeym.utils.validate_svg], verbose_name='Icon')),
                ('title', models.CharField(default='', max_length=60, unique=True, verbose_name='Title')),
                ('slug', models.SlugField(blank=True, max_length=255, null=True)),
            ],
            options={
                'verbose_name': 'Load indicator',
                'verbose_name_plural': 'Load indicators',
                'ordering': ('title', 'id'),
            },
        ),
        migrations.AddField(
            model_name='map',
            name='load_indicator_size',
            field=models.PositiveSmallIntegerField(choices=[(64, '64'), (96, '96'), (128, '128')], default=64, verbose_name='Load indicator size'),
        ),
        migrations.AddField(
            model_name='map',
            name='load_indicator',
            field=models.ForeignKey(blank=True, help_text='https://github.com/genkosta/django-editor-ymaps/blob/master/Spinner.zip', null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='map_load_indicator', to='djeym.LoadIndicator', verbose_name='Load indicator'),
        ),
    ]
