# Generated by Django 2.1.7 on 2019-03-22 07:58

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djeym', '0002_auto_20190321_0953'),
    ]

    operations = [
        migrations.AddField(
            model_name='map',
            name='disable_indicator_animation',
            field=models.BooleanField(default=False, verbose_name='Disable loading indicator animation'),
        ),
    ]
