# Generated by Django 2.1.7 on 2019-03-30 14:58

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djeym', '0003_map_disable_indicator_animation'),
    ]

    operations = [
        migrations.AddField(
            model_name='map',
            name='animation_speed',
            field=models.CharField(choices=[('0.3', '0.3'), ('0.4', '0.4'), ('0.5', '0.5'), ('0.6', '0.6'), ('0.7', '0.7'), ('0.8', '0.8'), ('0.9', '0.9'), ('1', '1'), ('1.1', '1.1'), ('1.2', '1.2'), ('1.3', '1.3'), ('1.4', '1.4'), ('1.5', '1.5'), ('1.6', '1.6'), ('1.7', '1.7'), ('1.8', '1.8')], default='0.8', max_length=255, verbose_name='Load indicator animation speed'),
        ),
        migrations.AlterField(
            model_name='map',
            name='disable_indicator_animation',
            field=models.BooleanField(default=False, help_text='It may be useful for the abbreviation or logo of the company, if it does not make sense to animate them.', verbose_name='Disable loading indicator animation'),
        ),
    ]
